@@ -31,15 +31,20 @@ try:
     from oslo_vmware import vim_util
 except ImportError:
     api = None
-from six.moves import http_client
-from six.moves import urllib
 
+from six.moves import urllib
+import six.moves.urllib.parse as urlparse
+
+import requests
+from requests import adapters
+from requests.packages.urllib3.util import retry
 import six
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
 from six.moves import range
 
 import glance_store
 from glance_store import capabilities
+from glance_store.common import utils
 from glance_store import exceptions
 from glance_store.i18n import _
 from glance_store.i18n import _LE
@@ -48,6 +53,7 @@ from glance_store import location
 
 LOG = logging.getLogger(__name__)
 
+CHUNKSIZE = 1024 * 64  # 64kB
 MAX_REDIRECTS = 5
 DEFAULT_STORE_IMAGE_DIR = '/openstack_glance'
 DS_URL_PREFIX = '/folder'
@@ -67,21 +73,6 @@ _VMWARE_OPTS = [
                help=_('Password for authenticating with '
                       'VMware ESX/VC server.'),
                secret=True),
-    cfg.StrOpt('vmware_datacenter_path',
-               default='ha-datacenter',
-               help=_('DEPRECATED. Inventory path to a datacenter. '
-                      'If the vmware_server_host specified is an ESX/ESXi, '
-                      'the vmware_datacenter_path is optional. If specified, '
-                      'it should be "ha-datacenter". This option is '
-                      'deprecated in favor of vmware_datastores and will be '
-                      'removed in the Liberty release.'),
-               deprecated_for_removal=True),
-    cfg.StrOpt('vmware_datastore_name',
-               help=_('DEPRECATED. Datastore associated with the datacenter. '
-                      'This option is deprecated in favor of '
-                      'vmware_datastores and will be removed in the Liberty '
-                      'release.'),
-               deprecated_for_removal=True),
     cfg.IntOpt('vmware_api_retry_count',
                default=10,
                help=_('Number of times VMware ESX/VC server API must be '
@@ -94,19 +85,25 @@ _VMWARE_OPTS = [
                default=DEFAULT_STORE_IMAGE_DIR,
                help=_('The name of the directory where the glance images '
                       'will be stored in the VMware datastore.')),
-    cfg.BoolOpt('vmware_api_insecure',
+    cfg.BoolOpt('vmware_insecure',
                 default=False,
-                help=_('Allow to perform insecure SSL requests to ESX/VC.')),
+                help=_('If true, the ESX/vCenter server certificate is not '
+                       'verified. If false, then the default CA truststore is '
+                       'used for verification. This option is ignored if '
+                       '"vmware_ca_file" is set.'),
+                deprecated_name='vmware_api_insecure'),
+    cfg.StrOpt('vmware_ca_file',
+               help=_('Specify a CA bundle file to use in verifying the '
+                      'ESX/vCenter server certificate.')),
     cfg.MultiStrOpt(
         'vmware_datastores',
         help=_(
             'A list of datastores where the image can be stored. This option '
             'may be specified multiple times for specifying multiple '
-            'datastores. Either one of vmware_datastore_name or '
-            'vmware_datastores is required. The datastore name should be '
-            'specified after its datacenter path, seperated by ":". An '
-            'optional weight may be given after the datastore name, seperated '
-            'again by ":". Thus, the required format becomes '
+            'datastores. The datastore name should be specified after its '
+            'datacenter path, seperated by ":". An optional weight may be '
+            'given after the datastore name, seperated again by ":". Thus, '
+            'the required format becomes '
             '<datacenter_path>:<datastore_name>:<optional_weight>. When '
             'adding an image, the datastore with highest weight will be '
             'selected, unless there is not enough free space available in '
@@ -135,15 +132,18 @@ def http_response_iterator(conn, response, size):
 
 class _Reader(object):
 
-    def __init__(self, data):
+    def __init__(self, data, verifier=None):
         self._size = 0
         self.data = data
         self.checksum = hashlib.md5()
+        self.verifier = verifier
 
     def read(self, size=None):
         result = self.data.read(size)
         self._size += len(result)
         self.checksum.update(result)
+        if self.verifier:
+            self.verifier.update(result)
         return result
 
     @property
@@ -151,47 +151,6 @@ class _Reader(object):
         return self._size
 
 
-class _ChunkReader(_Reader):
-
-    def __init__(self, data, blocksize=8192):
-        self.blocksize = blocksize
-        self.current_chunk = b""
-        self.closed = False
-        super(_ChunkReader, self).__init__(data)
-
-    def read(self, size=None):
-        ret = b""
-        while size is None or size >= len(self.current_chunk):
-            ret += self.current_chunk
-            if size is not None:
-                size -= len(self.current_chunk)
-            if self.closed:
-                self.current_chunk = b""
-                break
-            self._get_chunk()
-        else:
-            ret += self.current_chunk[:size]
-            self.current_chunk = self.current_chunk[size:]
-        return ret
-
-    def _get_chunk(self):
-        if not self.closed:
-            chunk = self.data.read(self.blocksize)
-            chunk_len = len(chunk)
-            self._size += chunk_len
-            self.checksum.update(chunk)
-            if chunk:
-                if six.PY3:
-                    size_header = ('%x\r\n' % chunk_len).encode('ascii')
-                    self.current_chunk = b''.join((size_header, chunk,
-                                                   b'\r\n'))
-                else:
-                    self.current_chunk = b'%x\r\n%s\r\n' % (chunk_len, chunk)
-            else:
-                self.current_chunk = b'0\r\n\r\n'
-                self.closed = True
-
-
 class StoreLocation(location.StoreLocation):
     """Class describing an VMware URI.
 
@@ -259,6 +218,16 @@ class StoreLocation(location.StoreLocation):
         if ds_name:
             self.datastore_name = ds_name[0]
 
+    @property
+    def https_url(self):
+        """
+        Creates a https url that can be used to upload/download data from a
+        vmware store.
+        """
+        parsed_url = urlparse.urlparse(self.get_uri())
+        new_url = parsed_url._replace(scheme='https')
+        return urlparse.urlunparse(new_url)
+
 
 class Store(glance_store.Store):
     """An implementation of the VMware datastore adapter."""
@@ -275,7 +244,9 @@ class Store(glance_store.Store):
     def reset_session(self):
         self.session = api.VMwareAPISession(
             self.server_host, self.server_username, self.server_password,
-            self.api_retry_count, self.tpoll_interval)
+            self.api_retry_count, self.tpoll_interval,
+            cacert=self.ca_file,
+            insecure=self.api_insecure)
         return self.session
 
     def get_schemes(self):
@@ -294,22 +265,6 @@ class Store(glance_store.Store):
             raise exceptions.BadStoreConfiguration(
                 store_name='vmware_datastore', reason=msg)
 
-        if not (self.conf.glance_store.vmware_datastore_name
-                or self.conf.glance_store.vmware_datastores):
-            msg = (_("Specify at least 'vmware_datastore_name' or "
-                     "'vmware_datastores' option"))
-            LOG.error(msg)
-            raise exceptions.BadStoreConfiguration(
-                store_name='vmware_datastore', reason=msg)
-
-        if (self.conf.glance_store.vmware_datastore_name and
-                self.conf.glance_store.vmware_datastores):
-            msg = (_("Specify either 'vmware_datastore_name' or "
-                     "'vmware_datastores' option"))
-            LOG.error(msg)
-            raise exceptions.BadStoreConfiguration(
-                store_name='vmware_datastore', reason=msg)
-
     def configure(self, re_raise_bsc=False):
         self._sanity_check()
         self.scheme = STORE_SCHEME
@@ -318,7 +273,8 @@ class Store(glance_store.Store):
         self.server_password = self._option_get('vmware_server_password')
         self.api_retry_count = self.conf.glance_store.vmware_api_retry_count
         self.tpoll_interval = self.conf.glance_store.vmware_task_poll_interval
-        self.api_insecure = self.conf.glance_store.vmware_api_insecure
+        self.ca_file = self.conf.glance_store.vmware_ca_file
+        self.api_insecure = self.conf.glance_store.vmware_insecure
         if api is None:
             msg = _("Missing dependencies: oslo_vmware")
             raise exceptions.BadStoreConfiguration(
@@ -412,15 +368,7 @@ class Store(glance_store.Store):
         return ds_map
 
     def configure_add(self):
-        if self.conf.glance_store.vmware_datastores:
-            datastores = self.conf.glance_store.vmware_datastores
-        else:
-            # Backwards compatibility for vmware_datastore_name and
-            # vmware_datacenter_path.
-            datacenter_path = self.conf.glance_store.vmware_datacenter_path
-            datastore_name = self._option_get('vmware_datastore_name')
-            datastores = ['%s:%s:%s' % (datacenter_path, datastore_name, 0)]
-
+        datastores = self._option_get('vmware_datastores')
         self.datastores = self._build_datastore_weighted_map(datastores)
         self.store_image_dir = self.conf.glance_store.vmware_store_image_dir
 
@@ -461,7 +409,8 @@ class Store(glance_store.Store):
             return cookie.name + '=' + cookie.value
 
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None):
+    def add(self, image_id, image_file, image_size, context=None,
+            verifier=None):
         """Stores an image file with supplied identifier to the backend
         storage system and returns a tuple containing information
         about the stored image.
@@ -469,24 +418,23 @@ class Store(glance_store.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param verifier: An object used to verify signatures for images
         :retval tuple of URL in backing store, bytes written, checksum
                 and a dictionary with storage system specific information
-        :raises `glance.common.exceptions.Duplicate` if the image already
+        :raises: `glance.common.exceptions.Duplicate` if the image already
                 existed
                 `glance.common.exceptions.UnexpectedStatus` if the upload
                 request returned an unexpected status. The expected responses
                 are 201 Created and 200 OK.
         """
         ds = self.select_datastore(image_size)
+        image_file = _Reader(image_file, verifier)
+        headers = {}
         if image_size > 0:
-            headers = {'Content-Length': image_size}
-            image_file = _Reader(image_file)
+            headers.update({'Content-Length': image_size})
+            data = image_file
         else:
-            # NOTE (arnaud): use chunk encoding when the image is still being
-            # generated by the server (ex: stream optimized disks generated by
-            # Nova).
-            headers = {'Transfer-Encoding': 'chunked'}
-            image_file = _ChunkReader(image_file)
+            data = utils.chunkiter(image_file, CHUNKSIZE)
         loc = StoreLocation({'scheme': self.scheme,
                              'server_host': self.server_host,
                              'image_dir': self.store_image_dir,
@@ -496,42 +444,46 @@ class Store(glance_store.Store):
         # NOTE(arnaud): use a decorator when the config is not tied to self
         cookie = self._build_vim_cookie_header(True)
         headers = dict(headers)
-        headers['Cookie'] = cookie
-        conn_class = self._get_http_conn_class()
-        conn = conn_class(loc.server_host)
-        url = urllib.parse.quote('%s?%s' % (loc.path, loc.query))
+        headers.update({'Cookie': cookie})
+        session = new_session(self.api_insecure, self.ca_file)
+
+        url = loc.https_url
         try:
-            conn.request('PUT', url, image_file, headers)
+            response = session.put(url, data=data, headers=headers)
         except IOError as e:
+            # TODO(sigmavirus24): Figure out what the new exception type would
+            # be in requests.
             # When a session is not authenticated, the socket is closed by
             # the server after sending the response. http_client has an open
             # issue with https that raises Broken Pipe
             # error instead of returning the response.
             # See http://bugs.python.org/issue16062. Here, we log the error
             # and continue to look into the response.
-            msg = _LE('Communication error sending http %(method)s request'
+            msg = _LE('Communication error sending http %(method)s request '
                       'to the url %(url)s.\n'
                       'Got IOError %(e)s') % {'method': 'PUT',
                                               'url': url,
                                               'e': e}
             LOG.error(msg)
+            raise exceptions.BackendException(msg)
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception(_LE('Failed to upload content of image '
                                   '%(image)s'), {'image': image_id})
-        res = conn.getresponse()
-        if res.status == http_client.CONFLICT:
+
+        res = response.raw
+        if res.status == requests.codes.conflict:
             raise exceptions.Duplicate(_("Image file %(image_id)s already "
                                          "exists!") %
                                        {'image_id': image_id})
 
-        if res.status not in (http_client.CREATED, http_client.OK):
+        if res.status not in (requests.codes.created, requests.codes.ok):
             msg = (_LE('Failed to upload content of image %(image)s. '
                        'The request returned an unexpected status: %(status)s.'
                        '\nThe response body:\n%(body)s') %
                    {'image': image_id,
                     'status': res.status,
-                    'body': res.body})
+                    'body': getattr(res, 'body', None)})
             LOG.error(msg)
             raise exceptions.BackendException(msg)
 
@@ -567,16 +519,24 @@ class Store(glance_store.Store):
         :param location: `glance_store.location.Location` object, supplied
                         from glance_store.location.get_location_from_uri()
         """
-        return self._query(location, 'HEAD')[2]
+        conn = None
+        try:
+            conn, resp, size = self._query(location, 'HEAD')
+            return size
+        finally:
+            # NOTE(sabari): Close the connection as the request was made with
+            # stream=True.
+            if conn is not None:
+                conn.close()
 
     @capabilities.check
     def delete(self, location, context=None):
         """Takes a `glance_store.location.Location` object that indicates
         where to find the image file to delete
 
-        :location `glance_store.location.Location` object, supplied
+        :param location: `glance_store.location.Location` object, supplied
                   from glance_store.location.get_location_from_uri()
-        :raises NotFound if image does not exist
+        :raises: NotFound if image does not exist
         """
         file_path = '[%s] %s' % (
             location.store_location.datastore_name,
@@ -592,37 +552,66 @@ class Store(glance_store.Store):
             self.session.wait_for_task(delete_task)
         except vexc.FileNotFoundException:
             msg = _('Image file %s not found') % file_path
-            LOG.warn(msg)
+            LOG.warning(msg)
             raise exceptions.NotFound(message=msg)
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception(_LE('Failed to delete image %(image)s '
                                   'content.') % {'image': location.image_id})
 
-    def _query(self, location, method, depth=0):
-        if depth > MAX_REDIRECTS:
+    def _query(self, location, method):
+        session = new_session(self.api_insecure, self.ca_file)
+        loc = location.store_location
+        redirects_followed = 0
+        # TODO(sabari): The redirect logic was added to handle cases when the
+        # backend redirects http url's to https. But the store never makes a
+        # http request and hence this can be safely removed.
+        while redirects_followed < MAX_REDIRECTS:
+            conn, resp = self._retry_request(session, method, location)
+
+            # NOTE(sigmavirus24): _retry_request handles 4xx and 5xx errors so
+            # if the response is not a redirect, we can return early.
+            if not conn.is_redirect:
+                break
+
+            redirects_followed += 1
+
+            location_header = conn.headers.get('location')
+            if location_header:
+                if resp.status not in (301, 302):
+                    reason = (_("The HTTP URL %(path)s attempted to redirect "
+                                "with an invalid %(status)s status code.")
+                              % {'path': loc.path, 'status': resp.status})
+                    LOG.info(reason)
+                    raise exceptions.BadStoreUri(message=reason)
+                conn.close()
+                location = self._new_location(location, location_header)
+        else:
+            # NOTE(sigmavirus24): We exceeded the maximum number of redirects
             msg = ("The HTTP URL exceeded %(max_redirects)s maximum "
                    "redirects.", {'max_redirects': MAX_REDIRECTS})
             LOG.debug(msg)
             raise exceptions.MaxRedirectsExceeded(redirects=MAX_REDIRECTS)
+
+        content_length = int(resp.getheader('content-length', 0))
+
+        return (conn, resp, content_length)
+
+    def _retry_request(self, session, method, location):
         loc = location.store_location
         # NOTE(arnaud): use a decorator when the config is not tied to self
         for i in range(self.api_retry_count + 1):
             cookie = self._build_vim_cookie_header()
             headers = {'Cookie': cookie}
-            try:
-                conn = self._get_http_conn(method, loc, headers)
-                resp = conn.getresponse()
-            except Exception:
-                with excutils.save_and_reraise_exception():
-                    LOG.exception(_LE('Failed to access image %(image)s '
-                                      'content.') % {'image':
-                                                     location.image_id})
+            conn = session.request(method, loc.https_url, headers=headers,
+                                   stream=True)
+            resp = conn.raw
+
             if resp.status >= 400:
-                if resp.status == http_client.UNAUTHORIZED:
+                if resp.status == requests.codes.unauthorized:
                     self.reset_session()
                     continue
-                if resp.status == http_client.NOT_FOUND:
+                if resp.status == requests.codes.not_found:
                     reason = _('VMware datastore could not find image at URI.')
                     LOG.info(reason)
                     raise exceptions.NotFound(message=reason)
@@ -631,34 +620,35 @@ class Store(glance_store.Store):
                 LOG.debug(msg)
                 raise exceptions.BadStoreUri(msg)
             break
-        location_header = resp.getheader('location')
-        if location_header:
-            if resp.status not in (301, 302):
-                reason = (_("The HTTP URL %(path)s attempted to redirect "
-                            "with an invalid %(status)s status code.")
-                          % {'path': loc.path, 'status': resp.status})
-                LOG.info(reason)
-                raise exceptions.BadStoreUri(message=reason)
-            location_class = glance_store.location.Location
-            new_loc = location_class(location.store_name,
-                                     location.store_location.__class__,
-                                     uri=location_header,
-                                     image_id=location.image_id,
-                                     store_specs=location.store_specs)
-            return self._query(new_loc, method, depth + 1)
-        content_length = int(resp.getheader('content-length', 0))
+        return conn, resp
 
-        return (conn, resp, content_length)
+    def _new_location(self, old_location, url):
+        store_name = old_location.store_name
+        store_class = old_location.store_location.__class__
+        image_id = old_location.image_id
+        store_specs = old_location.store_specs
+        # Note(sabari): The redirect url will have a scheme 'http(s)', but the
+        # store only accepts url with scheme 'vsphere'. Thus, replacing with
+        # store's scheme.
+        parsed_url = urlparse.urlparse(url)
+        new_url = parsed_url._replace(scheme='vsphere')
+        vsphere_url = urlparse.urlunparse(new_url)
+        return glance_store.location.Location(store_name,
+                                              store_class,
+                                              self.conf,
+                                              uri=vsphere_url,
+                                              image_id=image_id,
+                                              store_specs=store_specs)
 
-    def _get_http_conn(self, method, loc, headers, content=None):
-        conn_class = self._get_http_conn_class()
-        conn = conn_class(loc.server_host)
-        url = urllib.parse.quote('%s?%s' % (loc.path, loc.query))
-        conn.request(method, url, content, headers)
 
-        return conn
-
-    def _get_http_conn_class(self):
-        if self.api_insecure:
-            return http_client.HTTPConnection
-        return http_client.HTTPSConnection
+def new_session(insecure=False, ca_file=None, total_retries=None):
+    session = requests.Session()
+    if total_retries is not None:
+        http_adapter = adapters.HTTPAdapter(
+            max_retries=retry.Retry(total=total_retries))
+        https_adapter = adapters.HTTPAdapter(
+            max_retries=retry.Retry(total=total_retries))
+        session.mount('http://', http_adapter)
+        session.mount('https://', https_adapter)
+    session.verify = ca_file if ca_file else not insecure
+    return session
