@@ -15,10 +15,13 @@
 
 """Glance Store exception subclasses"""
 
-from six.moves import urllib
+from debtcollector import removals
+import six
+import six.moves.urllib.parse as urlparse
+import warnings
 
 from glance_store import i18n
-
+warnings.simplefilter('always')
 _ = i18n._
 
 
@@ -32,7 +35,7 @@ class UnsupportedBackend(BackendException):
 
 class RedirectException(Exception):
     def __init__(self, url):
-        self.url = urllib.parse.urlparse(url)
+        self.url = urlparse.urlparse(url)
 
 
 class GlanceStoreException(Exception):
@@ -43,11 +46,24 @@ class GlanceStoreException(Exception):
     a 'message' property. That message will get printf'd
     with the keyword arguments provided to the constructor.
     """
-    message = ''
+    message = _("An unknown exception occurred")
 
-    def __init__(self, **kwargs):
-        self.msg = kwargs.pop('message', None) or self.message % kwargs
-        super(Exception, self).__init__(self.msg)
+    def __init__(self, message=None, **kwargs):
+        if not message:
+            message = self.message
+        try:
+            if kwargs:
+                message = message % kwargs
+        except Exception:
+                pass
+        self.msg = message
+        super(GlanceStoreException, self).__init__(message)
+
+    def __unicode__(self):
+        # NOTE(flwang): By default, self.msg is an instance of Message, which
+        # can't be converted by str(). Based on the definition of
+        # __unicode__, it should return unicode always.
+        return six.text_type(self.msg)
 
 
 class MissingCredentialError(GlanceStoreException):
@@ -79,7 +95,9 @@ class Duplicate(GlanceStoreException):
     message = _("Image %(image)s already exists")
 
 
+@removals.remove(version="0.10.0")
 class Conflict(GlanceStoreException):
+    """DEPRECATED!"""
     message = _("An object with the same identifier is currently being "
                 "operated on.")
 
@@ -112,15 +130,21 @@ class Forbidden(GlanceStoreException):
     message = _("You are not authorized to complete this action.")
 
 
+@removals.remove(version="0.10.0")
 class ForbiddenPublicImage(Forbidden):
+    """DEPRECATED!"""
     message = _("You are not authorized to complete this action.")
 
 
+@removals.remove(version="0.10.0")
 class ProtectedImageDelete(Forbidden):
+    """DEPRECATED!"""
     message = _("Image %(image_id)s is protected and cannot be deleted.")
 
 
 class Invalid(GlanceStoreException):
+    # NOTE(NiallBunting) This could be deprecated however the debtcollector
+    # seems to have problems deprecating this as well as the subclasses.
     message = _("Data supplied was not valid.")
 
 
@@ -133,7 +157,9 @@ class DriverLoadFailure(GlanceStoreException):
     message = _("Driver %(driver_name)s could not be loaded.")
 
 
+@removals.remove(version="0.10.0")
 class BadDriverConfiguration(GlanceStoreException):
+    """DEPRECATED!"""
     message = _("Driver %(driver_name)s could not be configured correctly. "
                 "Reason: %(reason)s")
 
@@ -160,7 +186,9 @@ class MaxRedirectsExceeded(GlanceStoreException):
     message = _("Maximum redirects (%(redirects)s) was exceeded.")
 
 
+@removals.remove(version="0.10.0")
 class InvalidRedirect(GlanceStoreException):
+    """DEPRECATED!"""
     message = _("Received invalid HTTP redirect.")
 
 
@@ -178,37 +206,55 @@ class RemoteServiceUnavailable(GlanceStoreException):
     message = _("Remote server where the image is present is unavailable.")
 
 
+@removals.remove(version="0.10.0")
 class WorkerCreationFailure(GlanceStoreException):
+    """DEPRECATED!"""
     message = _("Server worker creation failed: %(reason)s.")
 
 
+@removals.remove(version="0.10.0")
 class SchemaLoadError(GlanceStoreException):
+    """DEPRECATED!"""
     message = _("Unable to load schema: %(reason)s")
 
 
+@removals.remove(version="0.10.0")
 class InvalidObject(GlanceStoreException):
+    """DEPRECATED!"""
     message = _("Provided object does not match schema "
                 "'%(schema)s': %(reason)s")
 
 
+@removals.remove(version="0.10.0")
 class UnsupportedHeaderFeature(GlanceStoreException):
+    """DEPRECATED!"""
     message = _("Provided header feature is unsupported: %(feature)s")
 
 
+class HasSnapshot(GlanceStoreException):
+    message = _("The image cannot be deleted because it has snapshot(s).")
+
+
 class InUseByStore(GlanceStoreException):
     message = _("The image cannot be deleted because it is in use through "
                 "the backend store outside of Glance.")
 
 
+@removals.remove(version="0.10.0")
 class ImageDataNotFound(NotFound):
+    """DEPRECATED!"""
     message = _("No image data could be found")
 
 
+@removals.remove(version="0.10.0")
 class InvalidParameterValue(Invalid):
+    """DEPRECATED!"""
     message = _("Invalid value '%(value)s' for parameter '%(param)s': "
                 "%(extra_msg)s")
 
 
+@removals.remove(version="0.10.0")
 class InvalidImageStatusTransition(Invalid):
+    """DEPRECATED!"""
     message = _("Image status transition from %(cur_status)s to"
                 " %(new_status)s is not allowed")
