@@ -47,41 +47,24 @@ VMWARE_DS = {
     'vmware_server_host': '127.0.0.1',
     'vmware_server_username': 'username',
     'vmware_server_password': 'password',
-    'vmware_datacenter_path': 'dc1',
-    'vmware_datastore_name': 'ds1',
     'vmware_store_image_dir': '/openstack_glance',
-    'vmware_api_insecure': 'True',
+    'vmware_insecure': 'True',
+    'vmware_datastores': ['a:b:0'],
 }
 
 
-def format_location(host_ip, folder_name,
-                    image_id, datacenter_path, datastore_name):
+def format_location(host_ip, folder_name, image_id, datastores):
     """
     Helper method that returns a VMware Datastore store URI given
     the component pieces.
     """
     scheme = 'vsphere'
+    (datacenter_path, datastore_name, weight) = datastores[0].split(':')
     return ("%s://%s/folder%s/%s?dcPath=%s&dsName=%s"
             % (scheme, host_ip, folder_name,
                image_id, datacenter_path, datastore_name))
 
 
-class FakeHTTPConnection(object):
-
-    def __init__(self, status=200, *args, **kwargs):
-        self.status = status
-        pass
-
-    def getresponse(self):
-        return utils.FakeHTTPResponse(status=self.status)
-
-    def request(self, *_args, **_kwargs):
-        pass
-
-    def close(self):
-        pass
-
-
 def fake_datastore_obj(*args, **kwargs):
     dc_obj = oslo_datacenter.Datacenter(ref='fake-ref',
                                         name='fake-name')
@@ -107,9 +90,8 @@ class TestStore(base.StoreBaseTest,
                     vmware_server_username='admin',
                     vmware_server_password='admin',
                     vmware_server_host=VMWARE_DS['vmware_server_host'],
-                    vmware_api_insecure=VMWARE_DS['vmware_api_insecure'],
-                    vmware_datastore_name=VMWARE_DS['vmware_datastore_name'],
-                    vmware_datacenter_path=VMWARE_DS['vmware_datacenter_path'])
+                    vmware_insecure=VMWARE_DS['vmware_insecure'],
+                    vmware_datastores=VMWARE_DS['vmware_datastores'])
 
         mock_get_datastore.side_effect = fake_datastore_obj
         backend.create_stores(self.conf)
@@ -130,8 +112,8 @@ class TestStore(base.StoreBaseTest,
         loc = location.get_location_from_uri(
             "vsphere://127.0.0.1/folder/openstack_glance/%s"
             "?dsName=ds1&dcPath=dc1" % FAKE_UUID, conf=self.conf)
-        with self._mock_http_connection() as HttpConn:
-            HttpConn.return_value = FakeHTTPConnection()
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response()
             (image_file, image_size) = self.store.get(loc)
         self.assertEqual(image_size, expected_image_size)
         chunks = [c for c in image_file]
@@ -146,8 +128,8 @@ class TestStore(base.StoreBaseTest,
         loc = location.get_location_from_uri(
             "vsphere://127.0.0.1/folder/openstack_glan"
             "ce/%s?dsName=ds1&dcPath=dc1" % FAKE_UUID, conf=self.conf)
-        with self._mock_http_connection() as HttpConn:
-            HttpConn.return_value = FakeHTTPConnection(status=404)
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response(status_code=404)
             self.assertRaises(exceptions.NotFound, self.store.get, loc)
 
     @mock.patch.object(vm_store.Store, 'select_datastore')
@@ -168,11 +150,10 @@ class TestStore(base.StoreBaseTest,
                 VMWARE_DS['vmware_server_host'],
                 VMWARE_DS['vmware_store_image_dir'],
                 expected_image_id,
-                VMWARE_DS['vmware_datacenter_path'],
-                VMWARE_DS['vmware_datastore_name'])
+                VMWARE_DS['vmware_datastores'])
             image = six.BytesIO(expected_contents)
-            with self._mock_http_connection() as HttpConn:
-                HttpConn.return_value = FakeHTTPConnection()
+            with mock.patch('requests.Session.request') as HttpConn:
+                HttpConn.return_value = utils.fake_response()
                 location, size, checksum, _ = self.store.add(expected_image_id,
                                                              image,
                                                              expected_size)
@@ -203,11 +184,10 @@ class TestStore(base.StoreBaseTest,
                 VMWARE_DS['vmware_server_host'],
                 VMWARE_DS['vmware_store_image_dir'],
                 expected_image_id,
-                VMWARE_DS['vmware_datacenter_path'],
-                VMWARE_DS['vmware_datastore_name'])
+                VMWARE_DS['vmware_datastores'])
             image = six.BytesIO(expected_contents)
-            with self._mock_http_connection() as HttpConn:
-                HttpConn.return_value = FakeHTTPConnection()
+            with mock.patch('requests.Session.request') as HttpConn:
+                HttpConn.return_value = utils.fake_response()
                 location, size, checksum, _ = self.store.add(expected_image_id,
                                                              image, 0)
         self.assertEqual(utils.sort_url_by_qs_keys(expected_location),
@@ -215,18 +195,48 @@ class TestStore(base.StoreBaseTest,
         self.assertEqual(expected_size, size)
         self.assertEqual(expected_checksum, checksum)
 
+    @mock.patch.object(vm_store.Store, 'select_datastore')
+    @mock.patch('glance_store._drivers.vmware_datastore._Reader')
+    def test_add_with_verifier(self, fake_reader, fake_select_datastore):
+        """Test that the verifier is passed to the _Reader during add."""
+        verifier = mock.MagicMock(name='mock_verifier')
+        image_id = str(uuid.uuid4())
+        size = FIVE_KB
+        contents = b"*" * size
+        image = six.BytesIO(contents)
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response()
+            self.store.add(image_id, image, size, verifier=verifier)
+
+        fake_reader.assert_called_with(image, verifier)
+
+    @mock.patch.object(vm_store.Store, 'select_datastore')
+    @mock.patch('glance_store._drivers.vmware_datastore._Reader')
+    def test_add_with_verifier_size_zero(self, fake_reader, fake_select_ds):
+        """Test that the verifier is passed to the _ChunkReader during add."""
+        verifier = mock.MagicMock(name='mock_verifier')
+        image_id = str(uuid.uuid4())
+        size = FIVE_KB
+        contents = b"*" * size
+        image = six.BytesIO(contents)
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response()
+            self.store.add(image_id, image, 0, verifier=verifier)
+
+        fake_reader.assert_called_with(image, verifier)
+
     @mock.patch('oslo_vmware.api.VMwareAPISession')
     def test_delete(self, mock_api_session):
         """Test we can delete an existing image in the VMware store."""
         loc = location.get_location_from_uri(
             "vsphere://127.0.0.1/folder/openstack_glance/%s?"
             "dsName=ds1&dcPath=dc1" % FAKE_UUID, conf=self.conf)
-        with self._mock_http_connection() as HttpConn:
-            HttpConn.return_value = FakeHTTPConnection()
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response()
             vm_store.Store._service_content = mock.Mock()
             self.store.delete(loc)
-        with self._mock_http_connection() as HttpConn:
-            HttpConn.return_value = FakeHTTPConnection(status=404)
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response(status_code=404)
             self.assertRaises(exceptions.NotFound, self.store.get, loc)
 
     @mock.patch('oslo_vmware.api.VMwareAPISession')
@@ -250,8 +260,8 @@ class TestStore(base.StoreBaseTest,
         loc = location.get_location_from_uri(
             "vsphere://127.0.0.1/folder/openstack_glance/%s"
             "?dsName=ds1&dcPath=dc1" % FAKE_UUID, conf=self.conf)
-        with self._mock_http_connection() as HttpConn:
-            HttpConn.return_value = FakeHTTPConnection()
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response()
             image_size = self.store.get_size(loc)
         self.assertEqual(image_size, 31)
 
@@ -264,8 +274,8 @@ class TestStore(base.StoreBaseTest,
         loc = location.get_location_from_uri(
             "vsphere://127.0.0.1/folder/openstack_glan"
             "ce/%s?dsName=ds1&dcPath=dc1" % FAKE_UUID, conf=self.conf)
-        with self._mock_http_connection() as HttpConn:
-            HttpConn.return_value = FakeHTTPConnection(status=404)
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response(status_code=404)
             self.assertRaises(exceptions.NotFound, self.store.get_size, loc)
 
     def test_reader_full(self):
@@ -288,67 +298,13 @@ class TestStore(base.StoreBaseTest,
         self.assertEqual(expected_checksum, reader.checksum.hexdigest())
         self.assertEqual(1, reader.size)
 
-    def test_chunkreader_image_fits_in_blocksize(self):
-        """
-        Test that the image file reader returns the expected chunk of data
-        when the block size is larger than the image.
-        """
+    def test_reader_with_verifier(self):
         content = b'XXX'
         image = six.BytesIO(content)
-        expected_checksum = hashlib.md5(content).hexdigest()
-        reader = vm_store._ChunkReader(image)
-        ret = reader.read()
-        if six.PY3:
-            expected_chunk = ('%x\r\n%s\r\n'
-                              % (len(content), content.decode('ascii')))
-            expected_chunk = expected_chunk.encode('ascii')
-        else:
-            expected_chunk = b'%x\r\n%s\r\n' % (len(content), content)
-        last_chunk = b'0\r\n\r\n'
-        self.assertEqual(expected_chunk + last_chunk, ret)
-        self.assertEqual(len(content), reader.size)
-        self.assertEqual(expected_checksum, reader.checksum.hexdigest())
-        self.assertTrue(reader.closed)
-        ret = reader.read()
-        self.assertEqual(len(content), reader.size)
-        self.assertEqual(expected_checksum, reader.checksum.hexdigest())
-        self.assertTrue(reader.closed)
-        self.assertEqual(b'', ret)
-
-    def test_chunkreader_image_larger_blocksize(self):
-        """
-        Test that the image file reader returns the expected chunks when
-        the block size specified is smaller than the image.
-        """
-        content = b'XXX'
-        image = six.BytesIO(content)
-        expected_checksum = hashlib.md5(content).hexdigest()
-        last_chunk = b'0\r\n\r\n'
-        reader = vm_store._ChunkReader(image, blocksize=1)
-        ret = reader.read()
-        expected_chunk = b'1\r\nX\r\n'
-        expected = (expected_chunk + expected_chunk + expected_chunk
-                    + last_chunk)
-        self.assertEqual(expected,
-                         ret)
-        self.assertEqual(expected_checksum, reader.checksum.hexdigest())
-        self.assertEqual(len(content), reader.size)
-        self.assertTrue(reader.closed)
-
-    def test_chunkreader_size(self):
-        """Test that the image reader takes into account the specified size."""
-        content = b'XXX'
-        image = six.BytesIO(content)
-        expected_checksum = hashlib.md5(content).hexdigest()
-        reader = vm_store._ChunkReader(image, blocksize=1)
-        ret = reader.read(size=3)
-        self.assertEqual(b'1\r\n', ret)
-        ret = reader.read(size=1)
-        self.assertEqual(b'X', ret)
-        ret = reader.read()
-        self.assertEqual(expected_checksum, reader.checksum.hexdigest())
-        self.assertEqual(len(content), reader.size)
-        self.assertTrue(reader.closed)
+        verifier = mock.MagicMock(name='mock_verifier')
+        reader = vm_store._Reader(image, verifier)
+        reader.read()
+        verifier.update.assert_called_with(content)
 
     def test_sanity_check_api_retry_count(self):
         """Test that sanity check raises if api_retry_count is <= 0."""
@@ -381,16 +337,6 @@ class TestStore(base.StoreBaseTest,
     def test_sanity_check_multiple_datastores(self):
         self.store.conf.glance_store.vmware_api_retry_count = 1
         self.store.conf.glance_store.vmware_task_poll_interval = 1
-        # Check both vmware_datastore_name and vmware_datastores defined.
-        self.store.conf.glance_store.vmware_datastores = ['a:b:0']
-        self.assertRaises(exceptions.BadStoreConfiguration,
-                          self.store._sanity_check)
-        # Both vmware_datastore_name and vmware_datastores are not defined.
-        self.store.conf.glance_store.vmware_datastore_name = None
-        self.store.conf.glance_store.vmware_datastores = None
-        self.assertRaises(exceptions.BadStoreConfiguration,
-                          self.store._sanity_check)
-        self.store.conf.glance_store.vmware_datastore_name = None
         self.store.conf.glance_store.vmware_datastores = ['a:b:0', 'a:d:0']
         try:
             self.store._sanity_check()
@@ -441,8 +387,24 @@ class TestStore(base.StoreBaseTest,
         expected_contents = b"*" * expected_size
         image = six.BytesIO(expected_contents)
         self.session = mock.Mock()
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response(status_code=401)
+            self.assertRaises(exceptions.BackendException,
+                              self.store.add,
+                              expected_image_id, image, expected_size)
+
+    @mock.patch.object(vm_store.Store, 'select_datastore')
+    @mock.patch.object(api, 'VMwareAPISession')
+    def test_unexpected_status_no_response_body(self, mock_api_session,
+                                                mock_select_datastore):
+        expected_image_id = str(uuid.uuid4())
+        expected_size = FIVE_KB
+        expected_contents = b"*" * expected_size
+        image = six.BytesIO(expected_contents)
+        self.session = mock.Mock()
         with self._mock_http_connection() as HttpConn:
-            HttpConn.return_value = FakeHTTPConnection(status=401)
+            HttpConn.return_value = utils.fake_response(status_code=500,
+                                                        no_response_body=True)
             self.assertRaises(exceptions.BackendException,
                               self.store.add,
                               expected_image_id, image, expected_size)
@@ -482,7 +444,7 @@ class TestStore(base.StoreBaseTest,
         expected_contents = b"*" * expected_size
         image = six.BytesIO(expected_contents)
         self.session = mock.Mock()
-        with self._mock_http_connection() as HttpConn:
+        with mock.patch('requests.Session.request') as HttpConn:
             HttpConn.request.side_effect = IOError
             self.assertRaises(exceptions.BackendException,
                               self.store.add,
@@ -610,3 +572,58 @@ class TestStore(base.StoreBaseTest,
             'FindByInventoryPath',
             self.store.session.vim.service_content.searchIndex,
             inventoryPath=datacenter_path)
+
+    @mock.patch('oslo_vmware.api.VMwareAPISession')
+    def test_http_get_redirect(self, mock_api_session):
+        # Add two layers of redirects to the response stack, which will
+        # return the default 200 OK with the expected data after resolving
+        # both redirects.
+        redirect1 = {"location": "https://example.com?dsName=ds1&dcPath=dc1"}
+        redirect2 = {"location": "https://example.com?dsName=ds2&dcPath=dc2"}
+        responses = [utils.fake_response(),
+                     utils.fake_response(status_code=302, headers=redirect1),
+                     utils.fake_response(status_code=301, headers=redirect2)]
+
+        def getresponse(*args, **kwargs):
+            return responses.pop()
+
+        expected_image_size = 31
+        expected_returns = ['I am a teapot, short and stout\n']
+        loc = location.get_location_from_uri(
+            "vsphere://127.0.0.1/folder/openstack_glance/%s"
+            "?dsName=ds1&dcPath=dc1" % FAKE_UUID, conf=self.conf)
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.side_effect = getresponse
+            (image_file, image_size) = self.store.get(loc)
+        self.assertEqual(image_size, expected_image_size)
+        chunks = [c for c in image_file]
+        self.assertEqual(expected_returns, chunks)
+
+    @mock.patch('oslo_vmware.api.VMwareAPISession')
+    def test_http_get_max_redirects(self, mock_api_session):
+        redirect = {"location": "https://example.com?dsName=ds1&dcPath=dc1"}
+        responses = ([utils.fake_response(status_code=302, headers=redirect)]
+                     * (vm_store.MAX_REDIRECTS + 1))
+
+        def getresponse(*args, **kwargs):
+            return responses.pop()
+
+        loc = location.get_location_from_uri(
+            "vsphere://127.0.0.1/folder/openstack_glance/%s"
+            "?dsName=ds1&dcPath=dc1" % FAKE_UUID, conf=self.conf)
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.side_effect = getresponse
+            self.assertRaises(exceptions.MaxRedirectsExceeded, self.store.get,
+                              loc)
+
+    @mock.patch('oslo_vmware.api.VMwareAPISession')
+    def test_http_get_redirect_invalid(self, mock_api_session):
+        redirect = {"location": "https://example.com?dsName=ds1&dcPath=dc1"}
+
+        loc = location.get_location_from_uri(
+            "vsphere://127.0.0.1/folder/openstack_glance/%s"
+            "?dsName=ds1&dcPath=dc1" % FAKE_UUID, conf=self.conf)
+        with mock.patch('requests.Session.request') as HttpConn:
+            HttpConn.return_value = utils.fake_response(status_code=307,
+                                                        headers=redirect)
+            self.assertRaises(exceptions.BadStoreUri, self.store.get, loc)
